﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.CombinedIntegerVectorEncoding {
  /// <summary>
  /// A base class for operators creating combined int-valued vectors.
  /// </summary>
  [Item("CombinedIntegerVectorCreator", "A base class for operators creating combined int-valued vectors.")]
  [StorableClass]
  public abstract class CombinedIntegerVectorCreator : IntegerVectorCreator, ICombinedIntegerVectorCreator {
    public IValueLookupParameter<IntValue> ActionPartLengthParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["ActionPartLength"]; }
    }

    public ILookupParameter<CombinedIntegerVector> CombinedIntegerVectorParameter {
      get { return (ILookupParameter<CombinedIntegerVector>)Parameters["CombinedIntegerVector"]; }
    }

    [StorableConstructor]
    protected CombinedIntegerVectorCreator(bool deserializing) : base(deserializing) { }
    protected CombinedIntegerVectorCreator(IntegerVectorCreator original, Cloner cloner) : base(original, cloner) { }
    protected CombinedIntegerVectorCreator()
      : base() {
      Parameters.Add(new LookupParameter<CombinedIntegerVector>("CombinedIntegerVector", "The vector which should be manipulated."));
      Parameters.Add(new ValueLookupParameter<IntValue>("ActionPartLength", "The length of the vector."));
    }

    protected override IntegerVector Create(IRandom random, IntValue length, IntMatrix bounds) {
      CombinedIntegerVectorParameter.ActualValue = Create(random, length, bounds, ActionPartLengthParameter.ActualValue);
      return CombinedIntegerVectorParameter.ActualValue;
    }

    protected abstract CombinedIntegerVector Create(IRandom random, IntValue length, IntMatrix bounds, IntValue actionPartLength);
  }
}
